#!/usr/bin/perl

package Nagios::Command::Memcached;

use strict;
use warnings;
use Cache::Memcached::Fast;
use Digest::MD5 qw//;

our $NAGIOS_CHECK_KEY = "_n_a_g_i_o_s_";
our $NAGIOS_EXPIRES   = 60 * 10;

sub new {
    my $class = shift;
    bless {@_}, $class;
}

sub memcached {
    my $self = shift;
    $self->{_memcached} ||= Cache::Memcached::Fast->new({
        servers => ["$self->{host}:$self->{port}"]
    });
    $self->{_memcached}
}

sub version {
    my $self = shift;
    my $version = $self->memcached->server_versions();
    die "could not get version, probably connection fail" unless $version->{"$self->{host}:$self->{port}"};
}

sub previous_key {
    my $self = shift;
    my $previous_timestamp = $self->memcached->get($NAGIOS_CHECK_KEY);
    die "could not get previous_timestamp\n" unless $previous_timestamp;
    my $previous_digest = $self->memcached->get($NAGIOS_CHECK_KEY .':' . $previous_timestamp);
    die "could not get previous_digest\n" unless $previous_digest;
    die "previous_digest incorrect\n" if $previous_digest ne $self->digest( $previous_timestamp );
    1;
}

sub next_key {
    my $self = shift;
    my $next_time = time;
    my $next_timestamp = $self->memcached->set($NAGIOS_CHECK_KEY, $next_time, 0);
    die "could not set next timestamp\n" unless $next_timestamp;
    my $next_digest = $self->memcached->add(
        $NAGIOS_CHECK_KEY. ':'. $next_time,
        $self->digest($next_time),
        $NAGIOS_EXPIRES
    );
    die "could not set next digest\n" unless $next_digest;
    1;
}

sub digest {
    my $self = shift;
    my $timestamp = shift;
    Digest::MD5::md5_hex( $timestamp . $NAGIOS_CHECK_KEY );
}

1;

package main;

use warnings;
use strict;
use Getopt::Long;

my $INTERVAL = 2*60*60;
my $LIMIT    = 1000;
GetOptions(
    'H=s'    => \my $HOST,
    'p=s' => \my $PORT,
);

if ( !$HOST ) {
    print "host not found";
    exit(3);
}
$PORT ||= 11211;


my $command = Nagios::Command::Memcached->new(
    host => $HOST,
    port => $PORT
);

my $error;

eval {
    $command->version();
};
$error ||= $@ if $@;

eval {
    $command->previous_key();
};
$error ||= $@ if $@;

eval {
    $command->next_key();
};
$error ||= $@ if $@;

if ( $error ) {
    print $error;
    exit(2);
}

print "OK\n";
exit(0);

