#!/usr/bin/perl

use strict;
use warnings;
use YAML;
use Template;
use Getopt::Long;
use FindBin;

$|=1;
my @argv = @ARGV;

GetOptions(
    'c|config=s'                   => \my $conf,
);

$conf ||= "$FindBin::Bin/service.yaml";

my $config = YAML::LoadFile($conf);

my $targets = $config->{targets};
my $output = '';
my @hosts;

foreach my $target ( @$targets ) {

    my $template = "$FindBin::Bin/tmpl/$target->{template}";
    my $tt = Template->new(
        ABSOLUTE => 1,
    );

    foreach my $host ( @{$target->{hosts}} ) {
        my ($ip, $hostname, $arg1, $desc) = split /\s+/, $host, 4;
        my $description = join " ", grep {$_} ($arg1, $desc);
        my $output_host = '';
        $tt->process(
            $template,
            {
                ip => $ip,
                hostname => $hostname,
                description => $description,
                arg1        => $arg1
            },
            \$output_host
        );
        push @hosts, $hostname;
        $output .= $output_host;
    }

}

my $hosts = join ",", @hosts;

print <<EOF;
define hostgroup {
hostgroup_name                 mixi-servers
alias                          mixi-servers
members $hosts
}

$output
EOF

